# $Id: IPmanage.pm 112 2007-08-03 15:37:28Z gwolf $

package IPmanage;

=head1 NAME

IPmanage - 

=head1 SYNOPSIS

  new
  process
  main_menu
  ip_data
  list
  passwd
  sys_par
  content_type
  get_output
  session
  reget_params

=head1 SEE ALSO

=head1 AUTHOR

Gunnar Wolf <gwolf@gwolf.org>

=head1 COPYRIGHT

Copyright 2005 Gunnar Wolf / Instituto de Investigaciones Econmicas UNAM
This module is Free Software, it can be redistributed under the same terms
as Perl.

=cut

use warnings;
use strict;
use Carp;
use DBI;
use IPmanage::Conf;
use IPmanage::IP;
use IPmanage::IPlist;
use IPmanage::Networks;
use IPmanage::Notif;
use Net::ARP;
use User::Simple;
use User::Simple::Admin;
use Template;
use Data::Dumper;

######################################################################
# Object creation / dispatcher

sub new {
    my ($self, $class, %init, @dsn, $res, $lang);
    $class = shift;
    %init = @_;

    $self = { };
    $res = '';
    bless $self, $class;

    # Are we debugging?
    $self->{debug} = $init{debug};

    # Put in our object the parameters sent by the client, the action 
    # requested (taken from the path).
    # Sometimes the reference to the base URL gathers ugly extra trailing 
    # slashes - Just strip them.
    $self->{params} = $init{params};
    $self->{action} = $init{action};
    $self->{base} = $init{uri_base};
    $self->{base} =~ s!/+$!/!;

    # Establish a DB connection
    @dsn = ("dbi:Pg:dbname=$init{dbname}", $init{dbuser}, $init{dbpass},
	    {AutoCommit => 1, PrintError => 1, RaiseError => 0,
	     ShowErrorStatement => 1});
    $dsn[0] .= ";host=$init{dbhost}" if defined $init{dbhost};
    unless ($self->{db} = DBI->connect(@dsn)) {
	carp 'Could not connect to the database';
	return undef;
    }

    # Set the defaults for the process result and the default content-type
    $self->{res} = \$res;
    $self->{content_type} = 'text/html';

    # Initialize Template Toolkit with the specified language file
    # If no language file is specified or it does not exist, the system will
    # be kind-of-usable anyway... But we should at least send a warning.
    $lang = "lang/$init{lang}";
    unless (-f "$init{tmpldir}/$lang") {
	warn "Requested language file ($init{tmpldir}/$lang) was not found\n";
	warn "Entering IPmanage with no language. This will not be pretty.\n";

	# Keep Template from panicing
	undef $lang;
    }
    $self->{tt} = Template->new(INCLUDE_PATH => $init{tmpldir},
				PRE_PROCESS => $lang,
				OUTPUT => $self->{res}) or 
				    die Template->error;

    # User validation
    unless ($self->{user} = User::Simple->new(db => $self->{db}, 
					      tbl =>'user_simple',
					      durat => 30,
					      debug => 2)) {
	carp 'Could not create User::Simple object';
	return undef;
    }

    # User::Admin - It always comes in handy for querying on user data! ;-)
    unless ($self->{user_adm} = User::Simple::Admin->new($self->{db},
							 'user_simple')) {
	carp 'Could not create User::Simple::Admin object';
	return undef;
    }

    # Get a IPmanage::Notif module ready to be called
    unless ($self->{notif} = IPmanage::Notif->new($self->{db})) {
	carp "Could not create IPmanage::Notif object";
	return undef;
    }

    # Get the client's IP address. If we didn't receive it, send a warning.
    $self->{remote_ip} = $init{remote_ip} or
	warn 'Remote IP was not specified';
    # Try to find out its associated MAC address.
    # Yes, Net::ARP's invocation syntax is _weird_ - The result goes into
    # $self->{remote_mac}
    $self->{remote_mac} = '';
    Net::ARP::arp_lookup($init{iface},$self->{remote_ip}, $self->{remote_mac});

    if ($self->{params}{login} and $self->{params}{passwd}) {
	# We have a login/password pair
	$self->{user}->ck_login($self->{params}{login},
				$self->{params}{passwd});
    } elsif ($init{session}) {
	# We have a session
	$self->{user}->ck_session($init{session});
    }

    return $self;
}

sub process {
    my ($self, $tmpl_file, %params);
    $self = shift;

    # Initialize the template defaults:
    #   %params: Hash with the values to be substituted
    #   $tmpl_file: Template filename to use by default
    $tmpl_file = 'list.tmpl';
    %params = (base => $self->{base},
	       client_ip => $self->{remote_ip},
	       client_mac => $self->{remote_mac},
	       title => '',
	       msg => [],
	       err => []
	       );

    # Do we have a valid user, or the request comes from an pre-authorized IP?
    unless ($self->{user}->is_valid or $self->ip_can_do($self->{action}[0])) {
	# No - Ask for a login, carrying only the bare parameters
	$self->{tt}->process('login.tmpl', \%params);
	return 1;
    }

    # Ok, the user is valid - Now populate the template with meaningful data
    $params{path} = $self->{action};
    $params{menu} = $self->main_menu;

    $params{user} = {login => $self->{user}->login,
		     name => $self->{user}->name} if $self->{user}->is_valid;

    if (! $self->{action}[0] or $self->{action}[0] eq 'list') {
	$tmpl_file = 'list.tmpl';
	$self->list(\%params);

    } elsif ($self->{action}[0] eq 'gen_dhcpd_conf') {
	$tmpl_file = 'dhcpd.conf.tmpl';
	$self->{content_type} = 'text/plain';
	$self->gen_dhcpd_conf(\%params);

    } elsif ($self->{action}[0] eq 'gen_dns') {
	$tmpl_file = 'dns.tmpl';
	$self->{content_type} = 'text/plain';
	$self->gen_dns(\%params);

    } elsif ($self->{action}[0] eq 'gen_rev_dns' and 
	     my $net = $self->{action}[1]) {
	$tmpl_file = 'reverse_dns.tmpl';
	$self->{content_type} = 'text/plain';
	$self->gen_rev_dns(\%params, $net);

    } elsif ($self->{action}[0] eq 'ip_data') {
	$tmpl_file = 'ip_data.tmpl';
	$self->ip_data(\%params);

    } elsif ($self->{action}[0] eq 'search') {
	my ($nets, %users);
	$tmpl_file = 'search.tmpl';
	$nets = IPmanage::Networks->new($self->{db});
	%users = $self->{user_adm}->dump_users;

	$params{networks} = [ $nets->list ];
	$params{mod_users} = [ map {{id => $_, name => $users{$_}{name}}}
			       sort {$users{$a}{name} cmp $users{$b}{name}}
			       keys %users];

    } elsif ($self->{action}[0] eq 'passwd') {
	$self->passwd(\%params, \$tmpl_file);

    } elsif ($self->{user}->level and $self->{action}[0] eq 'sys_par') {
	$tmpl_file = 'sys_par.tmpl';
	$self->sys_par(\%params);

    } elsif ($self->{user}->level and $self->{action}[0] eq 'net_mgmt') {
	$tmpl_file = 'net_mgmt.tmpl';
	$self->net_mgmt(\%params);

    } elsif ($self->{user}->level and $self->{action}[0] eq 'user_mgmt') {
	$tmpl_file = 'user_mgmt.tmpl';
	$self->user_mgmt(\%params);

    } elsif ($self->{action}[0] eq 'end') {
	$self->{user}->end_session;
	$tmpl_file = 'end.tmpl';

    }
    $self->{tt}->process($tmpl_file, \%params)
	or die $self->{tt}->error;

    return 1;
}

######################################################################
# Interface elements

sub main_menu {
    my ($self);
    $self = shift;

    if ($self->{user}->level) {
	return [ map {{url => $_}} qw(list search sys_par net_mgmt user_mgmt 
				      passwd end) ];
    } else {
	return [ map {{url => $_}} qw(list search passwd end) ];
    }
}

sub ip_data {
    my ($self, $tmpl, $ip, $nets, %data);
    $self = shift;
    $tmpl = shift;

    $ip = IPmanage::IP->new($self->{db}, $self->{action}[1]);
    $nets = IPmanage::Networks->new($self->{db});

    if ($ip->is_gateway) {
	$tmpl->{is_gateway} = 1;
	return 1;
    }
    if ($nets->is_dynamic('ip', $ip->ip)) {
	$tmpl->{is_dynamic} = 1;
	return 1;
    }

    if (defined $self->{params}{modif}) {
	my ($err);
	$err = 0;

	# Part of the order in the following parameter list is in fact
	# important: blocked is triggered to be true in the DB in case mac is
	# null, so if those two are reversed, stupid bugs arise.
	for my $par qw (mac details inventory responsible hostname location
			blocked passwd) {
	    my $method;
	    # Instead of coding an ugly dispatch table, just ask if $ip can
	    # change the desired value. Thanks, UNIVERSAL!
	    unless ($method = $ip->can("set_$par") and 
		    $ip->$method($self->{params}{$par})) {
		push(@{$tmpl->{err}}, 'ip_update_error');
		carp "$par could not be updated";
		$err++;
	    }
	}

	# It does not really matter if there is an error updating the 
	# modif_user_id, we can just ignore it.
	$ip->set_modif_user_id($self->{user}->id);

	$self->{notif}->notify;
	push(@{$tmpl->{msg}}, 'ip_update_ok') unless $err;
    }

    %data = $ip->get_data;

    $data{modif_user_name} = $self->{user_adm}->name($data{modif_user_id});
    $tmpl->{ip} = { %data };
    $tmpl->{block_vals} = [ {id => 0, descr => 'No'}, 
			    {id => 1, descr => => 'Yes' } ];

    return 1;
}

sub list {
    my ($self, $tmpl, $list, $nets, %search);
    $self = shift;
    $tmpl = shift;

    $list = IPmanage::IPlist->new($self->{db});

    $nets = IPmanage::Networks->new($self->{db});

    $tmpl->{params} = $self->reget_params('ord');

    if (defined $self->{params}{search}) {
	for my $par (keys %{$self->{params}}) {
	    next unless grep {$par eq $_} qw(ip mac location responsible
					     hostname inventory blocked
					     max_rev_date min_rev_date 
					     modif_user_id dynamic netaddr);
	    next unless (defined $self->{params}{$par} and 
			 $self->{params}{$par} ne '');
	    $search{$par} = $self->{params}{$par};
	}
    }

    $tmpl->{list} = $list->gen_list(order => $self->{params}{ord}, %search);
    $tmpl->{dyn_nets} = [grep {$_->{dynamic}} $nets->list];

    return 1;
}

sub gen_dhcpd_conf {
    my ($self, $tmpl, $conf, $nets, $list, @networks);
    $self = shift;
    $tmpl = shift;

    unless ($conf = IPmanage::Conf->new($self->{db})) {
	carp 'Could not instantiate required objects';
	return undef;
    }

    unless ($nets = IPmanage::Networks->new($self->{db})) {
	carp 'Could not instantiate networks object';
	return undef;
    }

    unless ($list = IPmanage::IPlist->new($self->{db})) {
	carp "Could not instantiate IPlist object";
    }

    # Get the required configuration keys into the template
    for my $key qw(domain_name default_lease max_lease netbios_node_type
		   netbios_server dns_server site_name) {
	$tmpl->{$key} = $conf->get_val("dhcp_$key");
    }

    @networks = $nets->list;

    for my $net (0..$#networks) {
	if ($networks[$net]{dynamic}) {
	    my ($subnet, $first, $last, $gw, $gw_on_border);

	    $subnet = Net::IP->new($networks[$net]{net});
	    $gw = $networks[$net]{gateway};

	    # Where does the dynamic range start? The first address is the 
	    # network address, so we take the second one. If that one is the
	    # gateway, we need to skip to the fifth (we reserve two of 
	    # addresses to be statically used by the administrator - probably
	    # as the host on which to set a captive portal or so).
	    # Same thing regarding the last one. 
	    # In case we found the gateway this way, set $gw_on_border to 1.
	    $first = $subnet + 1;
	    if ($first->ip eq $gw) {
		$first = $subnet + 4;
		$gw_on_border = 1;
	    }
	    $last = $subnet + ($subnet->size - 2);
	    if ($last->ip eq $gw) {
		$last = $subnet + ($subnet->size - 5);
		$gw_on_border = 1;
	    }

	    if ($gw_on_border) {
		# Good, the gateway is on the border - Specify just one range
		$networks[$net]{range1_first_ip} = $first->ip;
		$networks[$net]{range1_last_ip} = $last->ip;

 	    } else {
		# Ok, we need to specify two ranges, avoiding the gateway
		# We'll have to iterate through the whole range. Clumsy :-/
		# but that's what Net::IP gives us. And, of course, we leave 
		# our two IPs for the local administrator - Both, after the 
		# gateway. Of course, if we fall over the edge (i.e. the 
		# gateway is x.y.z.253 on a /24 network, and adding 2 hosts 
		# afterwards is impossible), don't declare the second range.
		my ($ip, $previous_ip, $next_ip);

		$networks[$net]{range1_first_ip} = $first->ip;
		$networks[$net]{range2_last_ip} = $last->ip;

		$previous_ip = $ip = $subnet;
		while (++$ip) {
		    if ($ip->ip ne $gw) {
			$previous_ip = $ip;
			next;
		    }

		    $networks[$net]{range1_last_ip} = $previous_ip->ip;
		    if ($next_ip = $ip+3 and $last->bincomp('gt', $next_ip)) {
			$networks[$net]{range2_first_ip} = $next_ip->ip;
		    } else {
			delete $networks[$net]{range2_last_ip};
		    }
		}
	    }


	} else {
	    my (@list, @hosts);

	    @list = @{ $list->gen_list(netaddr => $networks[$net]{net}) };

	    for my $host (@list) {
		push @hosts, {ip => $host->{ip}, 
			      mac => $host->{mac}, 
			      hostname => $host->{hostname},
			      blocked => $host->{blocked}, 
			      location => $host->{location}};
	    }

	    $networks[$net]{hosts} = \@hosts;
	}
    }

    $tmpl->{networks} = \@networks;

    return 1;
}

sub _gen_dns_common {
    my ($self, $tmpl, $list, $conf);
    $self = shift;
    $tmpl = shift;

    # DNS serial? Yup, epoch (seconds since 1970)
    $tmpl->{serial} = time;

    unless ($conf = IPmanage::Conf->new($self->{db})) {
	carp 'Could not instantiate required objects';
	return undef;
    }

    unless ($list = IPmanage::IPlist->new($self->{db})) {
	carp "Could not instantiate IPlist object";
	return undef;
    }

    # Get the required configuration keys into the template
    for my $key qw(ttl server domain_name refresh retry expire minimum){
	$tmpl->{$key} = $conf->get_val("dns_$key");
    }

    return ($conf, $list);
}

sub gen_dns {
    my ($self, $tmpl, $conf, $list);
    $self = shift;
    $tmpl = shift;

    ($conf, $list) = $self->_gen_dns_common($tmpl);
    unless ($conf and $list) {
	carp "Could not create required objects for DNS record generation";
	return undef;
    }

    $tmpl->{hosts} = $list->gen_list;

    return 1;
}

sub gen_rev_dns {
    my ($self, $tmpl, $net, $conf, $list, @nets, @list, @hosts);
    $self = shift;
    $tmpl = shift;
    $net = shift;

    ($conf, $list) = $self->_gen_dns_common($tmpl);
    unless ($conf and $list) {
	carp "Could not create required objects for reverse DNS generation";
	return undef;
    }

    $tmpl->{network} = $net;

    unless ($net =~ /^\d{1,3}\.\d{1,3}(?:\.\d{1,3})?$/) {
	carp "Reverse DNS records can only be requested for complete B \n" .
	    "or C class network - Network $net is not in the correct format.";
	$tmpl->{neterror} = 1;
	return undef;
    }

    @list = @{ $list->gen_list(netaddr => $net ) };
    for my $host (@list) {
	# We need the partial IPs - That is, the IPs minus the network portion,
	# and with the octets (if more than one) inverted. We don't just use 
	# IPlist's last_octet as would limit us to 24-bit networks.
	my $partial_ip = $host->{ip};
	$partial_ip =~ s/^$net\.?//;
	$partial_ip = join '.', reverse(split /\./, $partial_ip);

	push @hosts, {hostname => $host->{hostname}, 
		      partial_ip => $partial_ip,
		      net_id => $host->{net_id}};
    }

    $tmpl->{hosts} = \@hosts;

    return 1;
}

sub sys_par {
    my ($self, $tmpl, $conf, %keys, $key, $val);
    $self = shift;
    $tmpl = shift;

    unless ($conf = IPmanage::Conf->new($self->{db})) {
	carp 'Could not instantiate configuration object';
	return undef;
    }

    $key = lc($self->{params}{key});
    $val = $self->{params}{val};

    # Is there anything to do, or are we just reporting?
    if ($key) {
	unless ($key =~ /^[a-zA-Z0-9-_]+$/) {
	    push(@{$tmpl->{err}}, 'sys_par_invalid_key');
	    undef $self->{params}{modif};
	    undef $self->{params}{create};
	    undef $self->{params}{remove};
	}

	if ($self->{params}{modif}) {
	    # Any changes to the existing keys?
	    if ($conf->set_val($key, $val)) {
		push @{$tmpl->{msg}}, 'sys_par_modif_ok';
	    } else {
		push @{$tmpl->{err}}, 'sys_par_modif_error';
	    }

	} elsif ($self->{params}{create}) {
	    # Any new keys?
	    if ($conf->create_key($key, $val)) {
		push @{$tmpl->{msg}}, 'sys_par_create_ok';
	    } else {
		push @{$tmpl->{err}}, 'sys_par_create_error';
	    }

	} elsif ($self->{params}{remove}) {
	    # Any keys are getting deleted?
	    if ($conf->del_key($key)) {
		push @{$tmpl->{msg}}, 'sys_par_delete_ok';
	    } else {
		push @{$tmpl->{err}}, 'sys_par_delete_error';
	    }
	}
	$self->{notif}->notify;	
    }

    # Now get the list of values, sort them alphabetically and return.
    %keys = map {$_ => $conf->get_val($_)} $conf->get_conf_keys;
    $tmpl->{conf} = [map {{key => $_, 
			   val => $keys{$_},
			   basic => $conf->is_basic($_)}}
		     sort keys %keys];

    return 1;
}

sub net_mgmt {
    my ($self, $tmpl, $nets, $id);
    $self = shift;
    $tmpl = shift;

    unless ($nets = IPmanage::Networks->new($self->{db})) {
	carp 'Could not create IPmanage::Networks object';
	return undef;
    }

    # Anything to create, modify, delete? (only one action per request)
    if ($self->{params}{create}) {
	unless ($nets->create(name => $self->{params}{name}, 
			      gateway => $self->{params}{gateway},
			      net => $self->{params}{net},
			      dynamic => $self->{params}{dynamic})) {
	    push @{$tmpl->{err}}, 'net_mgmt_create_error',
		$nets->error;
	}
	$self->{notif}->notify;

    } elsif ($self->{params}{modif} and $id = $self->{action}[1]) {
	$nets->set_name($id, $self->{params}{name}) or 
	    push @{$tmpl->{err}}, 'net_mgmt_set_name_error';
	$nets->set_gateway($id, $self->{params}{gateway}) or
	    push @{$tmpl->{err}}, 'net_mgmt_set_gw_error';

	if ($self->{params}{dynamic}) {
	    # Does this network already have registered information?
	    # If so, notify that the static data will be ignored.
	    # We do not handle the specific warning here - go to the template.
	    $tmpl->{dyn_with_static} = $nets->has_static_info($id);

	    $nets->set_dynamic($id) or 
		push @{$tmpl->{err}}, 'net_mgmt_set_dyn_error'

	} else {
	    $nets->unset_dynamic($id) or 
		push @{$tmpl->{err}}, 'net_mgmt_set_static_error';

	}
	push @{$tmpl->{msg}}, 'net_mgmt_modif_ok';
	$self->{notif}->notify;

    } elsif ($self->{params}{delete} and $id = $self->{action}[1]) {
	# Delete a network - Ask for confirmation
	if ($self->{params}{confirm}) {
	    # Confirmation already received
	    $nets->delete(id => $id);
	    push @{$tmpl->{msg}}, 'net_mgmt_delete_ok';

	} else {
	    $tmpl->{networks} = [$nets->get_data(id => $id)];
	    $tmpl->{confirm_delete} = 1;
	    $tmpl->{has_data} = $nets->has_static_info($id);
	    return 1;
	}
	$self->{notif}->notify;
    }

    $tmpl->{main_area} = 1;
    $tmpl->{networks} = [ $nets->list ];

    return 1;
}

sub user_mgmt {
    my ($self, $tmpl, $ua, $id, %users);
    $self= shift;
    $tmpl = shift;

    $ua = $self->{user_adm};

    if ($self->{params}{create}) {
	# Create a new user with the specified data
	$ua->new_user(login => $self->{params}{usr_login}, 
		      name =>$self->{params}{usr_name},
		      passwd => $self->{params}{usr_pass}, 
		      level => $self->{params}{usr_level}) or
			  push @{$tmpl->{err}}, 'user_mgmt_add_error';
    } elsif ($self->{params}{delete} and $id = $self->{action}[1]) {
	# Remove an existing user
	$ua->remove_user($id) or 
	    push @{$tmpl->{err}}, 'user_mgmt_delete_error';

    } elsif ($self->{params}{modif} and $id = $self->{action}[1]) {
	# Modify a specific user's data
	$ua->set_login($id, $self->{params}{usr_login}) or
	    push @{$tmpl->{err}}, 'user_mgmt_set_login_error';
	$ua->set_name($id, $self->{params}{usr_name}) or
	    push @{$tmpl->{err}}, 'user_mgmt_set_name_error';
	if ($self->{params}{usr_pass}) {
	    $ua->set_passwd($id, $self->{params}{usr_pass}) or
		push @{$tmpl->{err}}, 'user_mgmt_set_pass_error';
	}
	if (exists $self->{params}{usr_level}) {
	    # Don't allow user to promote/demote himself
	    if ($self->{user}->id == $id) {
		push @{$tmpl->{err}}, 'user_mgmt_self_level_error';
	    } else {
		$ua->set_level($id, $self->{params}{usr_level}) or
		    push @{$tmpl->{err}}, 'user_mgmt_set_admin_error';
	    }

	}
    }

    %users = $ua->dump_users;

    $tmpl->{users} = [map({{ id => $_, %{$users{$_}} }} 
			  sort({$users{$a}{name} cmp $users{$b}{name}} 
			       keys(%users)))];

    return 1;
}

sub passwd {
    my ($self, $tmpl, $tmpl_file_ref);
    $self = shift;
    $tmpl = shift;
    $tmpl_file_ref = shift;

    # Have we got the new password?
    if ($self->{params}{change}) {
	my $user;
	$user = User::Simple->new(db => $self->{db},
				  tbl => 'user_simple');

	# Is the old password right?
	unless ($user->ck_login($self->{user}->login, 
				$self->{params}{currpass}, 1)) {
	    push @{$tmpl->{err}}, 'pass_invalid';
	    $$tmpl_file_ref = 'passwd.tmpl';
	    return undef;
	}

	# Did we get a new password? Is it identical to the
	# confirmation?
	unless ($self->{params}{newpass} and
		$self->{params}{newpass} eq $self->{params}{newpass2}) {
	    push @{$tmpl->{err}}, 'pass_not_match';
	    $$tmpl_file_ref = 'passwd.tmpl';
	    return undef;
	}

	# Ok, go ahead, change the password!
	unless ($user->set_passwd($self->{params}{newpass})) {
	    push @{$tmpl->{err}}, 'pass_change_error';
	    $$tmpl_file_ref = 'passwd.tmpl';
	    return undef;
	}

	# Finally, prepare the generic list and return with a success message.
	push @{$tmpl->{msg}}, 'pass_change_ok';
	$self->list($tmpl);

    } else {
	# Nope, we need to ask for it yet.
	$$tmpl_file_ref = 'passwd.tmpl';
    }

    return 1;
}

######################################################################
# Private methods

sub content_type {
    my $self = shift;
    return $self->{content_type} || 'text/html';
}

sub set_content_type {
    my ($self, $type) = @_;
    $self->{content_type} = $type;
}

sub get_output {
    my $self = shift;
    return ${$self->{res}};
}

sub session { my $self = shift; return $self->{user}->session; }

sub reget_params {
    my ($self, @exclude, %par);
    $self = shift;
    @exclude = @_;
    %par = %{$self->{params}};
    for my $param (@exclude) {
	delete $par{$param};
    }

    return join('&', map {"$_=$par{$_}"} keys %par);
}

sub ip_can_do {
    my ($self, $action, @valid, @server_ip, $conf, $ips);
    $self = shift;
    $action = shift || ''; # Avoid warnings - undef means ''

    # List of valid actions that can be anonymously called from a 
    # pre-authorized IP. Yes, hard-coded in here - But it should only be used
    # here anyway :-}
    @valid = qw(gen_dhcpd_conf gen_dns gen_rev_dns);

    # Easiest things come first: If the requested action is not within @valid,
    # return 0 right away.
    return 0 unless grep {$action eq $_} @valid;

    unless ($conf = IPmanage::Conf->new($self->{db})) {
	carp 'Could not instantiate IPmanage::Conf object';
	return undef;
    }

    # We might have no configured IPs for this - Avoid generating useless 
    # warnings here.
    $ips = $conf->get_val('server_ips') or return 0;
    @server_ip = split(/\s*,\s*/, $ips);

    # If the client IP does not match any of @server_ip, return 0.
    return 0 unless grep {$self->{remote_ip} eq $_} @server_ip;

    # Joy, joy, joy!
    return 1;
}

1;
# $Log$
# Revision 1.15  2005/10/25 17:01:57  gwolf
# - Cambio is_admin en el manejo de User::Simple por level (aunque seguro
#   habr que recambiarlo ms adelante)
# - Cargo los estilos del IIEc
#
# Revision 1.14  2005/06/29 00:18:21  gwolf
# Added support for hostnames
#
# Revision 1.13  2005/06/14 23:08:41  gwolf
# Added hostname handling
#
# Revision 1.12  2005/05/17 01:19:58  gwolf
# Allow for gateways not to be located on the network boundary on dynamic networks
#
# Revision 1.11  2005/05/10 05:04:20  gwolf
# Introducing... IPmanage::Notif, for all of your notification needs!
#
# Revision 1.10  2005/05/05 19:40:20  gwolf
# Implemented a 'modif_user_id' field in ip, killed the modif_log subsystem
# as it brought too much complexity and was an overkill
#
# Revision 1.9  2005/05/05 18:54:04  gwolf
# Added some logic regarding which IP is connecting - Can now allow specific
# actions for certain IPs without need for authentication
#
# Revision 1.8  2005/05/02 19:06:10  gwolf
# - All strings were replaced by their descriptors to make i18n possible
# - Regular user and administrative users now _are_ different
#
# Revision 1.7  2005/04/19 01:19:44  gwolf
# - More fine-tuning in network administration
# - Some more sanity checking in plpgsql
# - More network-related method (what was it? hmmm... Yup, checking if a
#   network has any static configurations set)
# - Firsts steps towards gen_dhcpd_conf
#
# Revision 1.6  2005/04/16 03:31:22  gwolf
# Functionality for managing networks. This is basically done, yay!
#
# Revision 1.5  2005/04/14 00:03:41  gwolf
# Continuing with the translation, it _seems_ everything is working as it should
#
# Revision 1.4  2005/04/06 23:04:35  gwolf
# Database changes (translation to English, removed some fields, details)
#
# Revision 1.3  2005/04/05 01:18:01  gwolf
# Translated to English.
# Have not tested functionality yet, but at least, it compiles. It deserves
# being shipped!
#
