# $Id: IP.pm 110 2007-04-24 22:40:06Z gwolf $

package IPmanage::IP;

=head1 NAME

IPmanage::IP - Handles an IP address' data for an IPmanage system

=head1 SYNOPSIS

  $ip = IPmanage::IP->new($db, $ip);
  $ip = IPmanage::IP->create($db, $ip);

  $ip_addr = $ip->ip;
  $locat = $ip->location;
  $resp = $ip->responsible;
  $host = $ip->hostname;
  $rev_date = $ip->rev_date;
  $uid = $ip->modif_user_id;
  $details = $ip->details;
  $mac = $ip->mac;
  $blocked = $ip->blocked;
  $inv = $ip->inventory;
  $passwd = $ip->passwd;

  $net = $ip->network;
  $gateway = $ip->gateway;
  $is_gw = $ip->is_gateway;

  $ok = $ip->set_location($locat);
  $ok = $ip->set_responsible($resp);
  $ok = $ip->set_hostname($host);
  $ok = $ip->set_details($details);
  $ok = $ip->set_mac($mac);
  $ok = $ip->set_inventory($inv);
  $ok = $ip->set_blocked($blocked);
  $ok = $ip->set_modif_user_id($uid);
  $ok = $ip->set_passwd($passwd);

  %data = $ip->get_data;
  $ok = $ip->refresh;
  $ok = IPmanage::IP->remove($db, $ip);

=head1 DESCRIPTION

=head2 Constructor

  $ip = IPmanage::IP->new($db, $ip);
  $ip = IPmanage::IP->create($db, $ip);

$db should be an open connection to a database (using DBI), $ip is the IP 
address we will work with, represented as a dotted quad string (i.e., 
C<$ip='192.168.10.15';>). 

C<new> will create a new IPmanage::IP object to work with an IP address which 
already exists in the database, C<create> will create a new entry with the 
specified address. 

Please note that C<create> will fail (and return undef) if the requested IP
address has already been registered, and C<new> will fail if it does not already
exist - You can either use C<IPmanage::IPlist> to check for this beforehand, or
use C<create> if C<new> fails.

=head2 Accessors / mutators

  $ip_addr = $ip->ip;
  $locat = $ip->location;
  $resp = $ip->responsible;
  $host = $ip->hostname;
  $rev_date = $ip->rev_date;
  $uid = $ip->modif_user_id;
  $details = $ip->details;
  $mac = $ip->mac;
  $inv = $ip->inventory;
  $blocked = $ip->blocked;

The value received from C<$ip-E<gt>blocked> is a boolean - A true value means
the IP address is blocked, a false value means it can be used.

  $network = $ip->network;
  $gateway = $ip->gateway;
  $is_gw = $ip->is_gateway;

The C<network> method returns the network to which this IP address belongs (as
an string). The C<gateway> method returns the gateway for this IP address (as 
a new L<IPmanage::IP> object), the C<is_gateway> is a shorthand for comparing 
this IP address to the gateway, returning 1 if they are equal and 0 otherwise. 

These three methods are presented separated from the other accessors as the 
gateway and the network are not a property of an IP address - They are 
linked to it, but they are common to many IP addresses, and thus can not be 
modified from within this module (see L<IPmanage::Networks> for working with 
them).

  $ok = $ip->set_location($locat);
  $ok = $ip->set_responsible($resp);
  $ok = $ip->set_hostname($host);
  $ok = $ip->set_details($details);
  $ok = $ip->set_mac($mac);
  $ok = $ip->set_inventory($inv);
  $ok = $ip->set_blocked($blocked);
  $ok = $ip->set_modif_user_id($uid)'

As for the data formats accepted: Location, responsible, details and inventory
are text fields in the database, so basically anything goes. 

Mac will accept a valid MAC address - six octets represented in binary, 
separated by either dashes (-) or colons (:), either at every byte boundary 
or at the center - This means, the following values are valid and equivalent:

  00DEAD:00BEEF
  00DEAD-00BEEF
  00:DE:AD:00:BE:EF
  00-DE-AD-00-BE-EF

Regarding the UID: IPmanage::IP will not check this UID is a valid one in the
system. Why? First of all, because user management is conceptually separated
from the database itself (see L<User::Simple> for user-related details), and
second, because users can be added or removed, and the IPs modified by them
should not disappear just because of that. C<$ip-E<gt>set_modif_user_id> will 
accept any integer, and C<$ip-E<gt>get_modif_user_id> will just return an 
integer.

You cannot change the IP address - You should delete this one and create a new
one for that. Sorry, C<IPmanage::IP> refers to a specific IP :)

=head2 Other functions

  %data = $ip->get_data;

Gets all of the object attributes (ip, location, responsible, hostname, 
rev_date, details, mac, blocked, inventory) in a single pass, returning a hash.

  $ok = $ip->refresh;

Refreshes the in-memory object, reading the values from the database.

  $ok = IPmanage::IP->delete($db, $ip);

Deletes the requested IP from the database. The expected parameters are in the
same format as in C<create> and C<new>.

=head1 SEE ALSO

L<IPmanage>, L<IPmanage::IPlist>, L<IPmanage::Networks>

=head1 AUTHOR

Gunnar Wolf <gwolf@gwolf.org>

=head1 COPYRIGHT

Copyright 2005 Gunnar Wolf / Instituto de Investigaciones Econmicas UNAM
This module is Free Software, it can be redistributed under the same terms 
as Perl.

=cut

use warnings;
use strict;
use Carp;
use IPmanage::Networks;

sub new {
    my ($self, $class, $db, $ip, $sth);
    $class = shift;
    $db = shift;
    $ip = shift;

    $self = { db => $db, ip => $ip };
    bless $self, $class;

    return $self->refresh ? $self : undef;
}

sub create {
    my ($self, $class, $db, $ip, $sth);
    $class = shift;
    $db = shift;
    $ip = shift;

    $sth = $db->prepare('INSERT INTO ip (ip) VALUES (?)');
    unless ($sth->execute($ip)) {
	carp "Could not create the requested address ($ip)";
	return undef;
    }

    $self = $class->new($db, $ip);
    bless $self, $class;
    return $self;
}

sub delete {
    my ($self, $class, $db, $ip, $sth);
    $class = shift;
    $db = shift;
    $ip = shift;

    $sth = $db->prepare('DELETE  FROM ip WHERE ip = ?');
    unless ($sth->execute($ip)) {
	carp "Could not delete the requested address ($ip)";
	return undef;
    }

    return 1;
}

sub refresh {
    my ($self, $sth, %val);
    $self = shift;

    $sth = $self->{db}->prepare('SELECT location, responsible, hostname, 
        rev_date, modif_user_id, details, inventory, mac, blocked, passwd
        FROM ip WHERE ip = ?');
    if ($sth->execute($self->ip) eq '0E0') {
	carp "Non-existant IP address ($self->{ip}) requested";
	return undef;
    }
    %val = %{$sth->fetchrow_hashref};

    map {$self->{$_} = $val{$_}} keys %val; 

    return 1;
}    

sub ip { my $self = shift; return $self->{ip}; }
sub location { my $self = shift; return $self->{location}; }
sub responsible { my $self = shift; return $self->{responsible}; }
sub hostname { my $self = shift; return $self->{hostname}; }
sub rev_date { my $self = shift; return $self->{rev_date}; }
sub modif_user_id { my $self = shift; return $self->{modif_user_id}; }
sub details { my $self = shift; return $self->{details}; }
sub mac { my $self = shift; return $self->{mac}; }
sub blocked { my $self = shift; return $self->{blocked}; }
sub inventory { my $self = shift; return $self->{inventory}; }
sub passwd { my $self = shift; return $self->{passwd}; }

sub network {
    my ($self, $nets);
    $self = shift;

    $nets = IPmanage::Networks->new($self->{db});

    return $nets->ip_belongs_to($self->ip);
}

sub gateway {
    my ($self, $nets, $data);
    $self = shift;

    $nets = IPmanage::Networks->new($self->{db});

    return $nets->get_gateway(ip => $self->ip);
}

sub is_gateway {
    my ($self, $ip, $gate);
    $self = shift;
    $ip = $self->ip || '-No IP-';
    $gate = $self->gateway || '-No gate-';

    return ($ip eq $gate) ? 1 : 0; 
}

sub get_data { 
    my ($self, @attr);
    $self = shift; 
    @attr = qw(ip location responsible hostname rev_date modif_user_id details
	       mac blocked inventory passwd);
    return map {$_ => $self->{$_}} @attr;
}

sub set_location {
    my ($self, $ubic);
    $self = shift;
    $ubic = shift;
    return $self->_set('location', $ubic);
}

sub set_responsible {
    my ($self, $resp);
    $self = shift;
    $resp = shift;
    return $self->_set('responsible', $resp);
}

sub set_hostname {
    my ($self, $host);
    $self = shift;
    # We sanitize a bit the hostname, to protect the innocent name servers from
    # non-ASCII characters ;-) (and, BTW, the underscore is forbidden as well)
    # It is quite incomplete right now - Blame it on my current keyboard, I 
    # cannot type the full set.
    $host = shift;
    $host =~ tr/_/aeiounAEIOUN-/;
    $host =~ s/[^A-Za-z0-9\-]//ig;
    return $self->_set('hostname', lc($host));
}

sub set_details {
    my ($self, $det);
    $self = shift;
    $det = shift;
    return $self->_set('details', $det);
}

sub set_mac {
    my ($self, $mac);
    $self = shift;
    # By having ||undef, it allows us to correctly set the MAC to NULL in the
    # database without sending any warnings
    $mac = shift || undef;
    return $self->_set('mac', $mac);
}

sub set_blocked {
    my ($self, $blocked);
    $self = shift;
    $blocked = shift;
    return $self->_set('blocked', $blocked);
}

sub set_inventory {
    my ($self, $inv);
    $self = shift;
    $inv = shift;
    return $self->_set('inventory', $inv);
}

sub set_passwd {
    my ($self, $passwd);
    $self = shift;
    $passwd = shift;
    return $self->_set('passwd', $passwd);
}

sub set_modif_user_id {
    my ($self, $uid);
    $self = shift;
    $uid = shift;
    return $self->_set('modif_user_id', $uid);
}

######################################################################
# Private methods

# _set receives a field name and the value to store in it. Modifies the
# current IP to have this value for this field in the DB;

sub _set {
    my ($self, $field, $value, $sth);
    $self = shift;
    $field = shift;
    $value = shift;

    return undef unless (defined $field and grep {$field eq $_} 
			 qw(location responsible hostname details mac blocked
			    inventory modif_user_id passwd));

    $sth = $self->{db}->prepare("UPDATE ip SET $field = ?, rev_date = now()
        WHERE ip = ?");
    return undef unless $sth->execute($value, $self->ip);

    $self->refresh;
}

1;

# $Log$
# Revision 1.10  2005/06/14 23:08:41  gwolf
# Added hostname handling
#
# Revision 1.9  2005/05/17 01:19:58  gwolf
# Allow for gateways not to be located on the network boundary on dynamic networks
#
# Revision 1.8  2005/05/05 19:40:20  gwolf
# Implemented a 'modif_user_id' field in ip, killed the modif_log subsystem
# as it brought too much complexity and was an overkill
#
# Revision 1.7  2005/05/02 19:07:15  gwolf
# Some extra methods for querying the IP's membership and role in a network
#
# Revision 1.6  2005/04/16 03:31:22  gwolf
# Functionality for managing networks. This is basically done, yay!
#
# Revision 1.5  2005/04/14 00:03:41  gwolf
# Continuing with the translation, it _seems_ everything is working as it should
#
# Revision 1.4  2005/04/06 23:04:35  gwolf
# Database changes (translation to English, removed some fields, details)
#
# Revision 1.3  2005/04/05 01:18:01  gwolf
# Translated to English.
# Have not tested functionality yet, but at least, it compiles. It deserves
# being shipped!
#
