# $Id: IPlist.pm 93 2006-07-28 15:59:49Z gwolf $

package IPmanage::IPlist;

=head1 NAME

IPmanage::IPlist - Handles lists of IP addresses

=head1 SYNOPSIS

  $ipl = IPmanage::IPlist->new($db);

  @list = $ipl->gen_list(order=>$order, 
                         ip=>$ip, 
                         mac=>$mac, 
                         location=>$loc,
                         responsible=>$resp,
                         hostname=>$host,
                         inventory=>$inv,
                         blocked=>$blocked, 
                         min_rev_date=>$date,
                         max_rev_date=>$date2, 
                         network=>$net, 
                         dynamic=>$dyn);

The generated list will have elements with the following format:

  { ip => $ip, 
    mac => $mac,
    hostname => $host,
    last_octet => $octet,
    netaddr => $netaddr, 
    gateway => $gateway,
    dynamic => $dynamic, 
    blocked => $blocked, 
    location => $location, 
    is_gateway => $is_gate }

Of course, all of the search fields must be in a valid representation of their
data types - C<ip> should be a valid IP address, C<mac> a valid MAC address,
C<last_octet> the last octet of the IP address (i.e. the host portion when 
dealing with 24-bit subnetworks), C<blocked> and C<dynamic> are boolean values
showing if said addresses should be blocked from Internet access and whether
they are part of a dynamic network, C<min_rev_date> and C<max_rev_date> dates,
and C<network> a CIDR networks specification.

C<is_gateway> is a boolean value, it will be true for the IP address which 
serves as the network's gateway - Probably it will deserve special treatment 
in the front-end.

=head1 AUTHOR

Gunnar Wolf <gwolf@gwolf.org>

=head1 COPYRIGHT

Copyright 2005 Gunnar Wolf / Instituto de Investigaciones Econmicas UNAM
This module is Free Software, it can be redistributed under the same terms 
as Perl.

=head1 SEE ALSO

L<IPmanage::IP>, to know what to do with this list ;-)

L<IPmanage>

=cut

use warnings;
use strict;

sub new {
    my ($self, $class, $db);
    $class = shift;
    $db = shift;
    $self = { db => $db };

    bless $self, $class;
    return $self;
}

sub gen_list {
    my ($self, %opt, %sql, $sth, @list);
    $self = shift;
    %opt = @_;
    @list = ();

    %sql = (base => 'SELECT * FROM ip JOIN network n ON ip.ip << n.netaddr',
	    where => [],
	    );

    # Ordering: We accept sorting by MAC or by IP. If the user requests 
    # ordering by anything other than the IP, make the results sub-ordered by
    # IP, as there id only unicity guarantee on that field
    $opt{order} ||= '';
    if ($opt{order} eq 'mac') {
	$sql{order} = 'mac, ip';

    } elsif ($opt{order} eq 'host') {
	$sql{order} = 'hostname, ip';

    } elsif ($opt{order} eq 'loc') {
	$sql{order} = 'lower(location), ip';

    } elsif ($opt{order} eq 'responsible') {
	$sql{order} = 'lower(responsible), ip';

    } else {
	$sql{order} = 'ip'
    }

    # Apply any received search criteria
    for my $field (keys %opt) {
	my ($nom_field, $oper);
	$oper = '=';

	next unless grep {$field eq $_} qw(ip mac location responsible hostname
					   inventory name blocked dynamic
					   min_rev_date max_rev_date 
					   modif_user_id netaddr);

	$oper = '~*' if grep {$field eq $_} qw(location responsible name 
	                                       hostname inventory);
	if ($field eq 'min_rev_date') {
	    $oper = '>';
	    $nom_field = 'ip.rev_date';

	} elsif ($field eq 'max_rev_date') {
	    $oper = '<';
	    $nom_field = 'ip.rev_date';

	} elsif ($field eq 'netaddr') {
	    $nom_field = 'n.netaddr';
	    $oper = '<<=';

	} elsif ($field eq 'dynamic') {
	    $nom_field = 'n.dynamic';

	} else {
	    $nom_field = "ip.$field";
	}

	push @{$sql{where}}, [$nom_field, $oper, $opt{$field}];
    }

    $sql{base} .= ' WHERE ' . join(' AND ', map {" $_->[0] $_->[1] ? "} 
				   @{$sql{where}}) if @{$sql{where}};

    $sql{base} .= " ORDER BY $sql{order} " if defined $sql{order};

    $sth = $self->{db}->prepare($sql{base});
    $sth->execute(map {$_->[2]} @{$sql{where}});

    while (my $data = $sth->fetchrow_hashref) {
	# The field name 'id' is not very descriptive - Rename it to 'net_id'
	$data->{net_id} = $data->{id};
	delete $data->{id};

	# The frontends require often just the last octet (i.e. to generate
	# DNS names for dynamic network's hosts). Put it in as well, just to
	# make everybody's life easier. Note that no warranty is given on
	# unicity (i.e. for <24 bits logical networks), this is just sometimes
	# useful.
	$data->{ip} =~ /\.(\d+)$/;
	$data->{last_octet} = $1;

	# The network's gateway probably deserves a special handling, but it
	# is up to the front-end to decide on it - we just report it with
	# is_gateway=>1
	$data->{is_gateway} = ($data->{ip} eq $data->{gateway}) ? 1 : 0;

	push @list, $data;
    }
    return \@list;
}

1;

# $Log$
# Revision 1.9  2005/06/29 00:18:21  gwolf
# Added support for hostnames
#
# Revision 1.8  2005/06/14 23:08:41  gwolf
# Added hostname handling
#
# Revision 1.7  2005/05/05 19:40:20  gwolf
# Implemented a 'modif_user_id' field in ip, killed the modif_log subsystem
# as it brought too much complexity and was an overkill
#
# Revision 1.6  2005/05/02 19:09:46  gwolf
# - Fixed some details regarding how ordering was specified.
# - Now includes information regarding a network's gateway
#
# Revision 1.5  2005/04/16 03:31:22  gwolf
# Functionality for managing networks. This is basically done, yay!
#
# Revision 1.4  2005/04/14 00:03:41  gwolf
# Continuing with the translation, it _seems_ everything is working as it should
#
# Revision 1.3  2005/04/06 23:04:35  gwolf
# Database changes (translation to English, removed some fields, details)
#
# Revision 1.2  2005/04/05 01:18:01  gwolf
# Translated to English.
# Have not tested functionality yet, but at least, it compiles. It deserves
# being shipped!
#
