# $Id: Notif.pm 81 2005-06-29 00:08:21Z gwolf $

package IPmanage::Notif;

=head1 NAME

IPmanage::Notif - Notifies servers about a configuration change

=head1 SYNOPSIS

  $notif = IPmanage::Notif->new($db);
  $ok = $notif->notify;

=head1 DESCRIPTION

  $notif = IPmanage::Notif->new($db);

Creates a new IPmanage::Notif object. Receives an open DBI connection as its 
only parameter.

  $ok = $notif->notify;

Signals the servers (as defined in the C<server_notif_addr> key of the IPmanage
configuration) they should get the new configuration files. 

Note that we cannot know how successful this notification was - We can only
know that we asked the server to get the new configuration, but we cannot 
assure success in the process. $ok will only be false if the connection to any
of the servers could not be established.

=head2 Details

How does the notification work?

When signalled to connect to the servers, this module opens a TCP connections 
to each of the addresses specified in the C<server_notif_addr> configuration
key. This key should be in the format:

  IP_addr1:port1,IP_addr2:port2,(etc)

This means, pairs of IP addresses and TCP ports (delimited by a colon) 
separated by commas.

Once connected to the server, we wait for a string which will be appended to 
the password provided at the object creation time. We get the MD5sum of this
value, and send it back to the server to identify as a legitimate client. The
server should then connect to the IPmanage HTTP interface to retreive the new
configuration files.

=head1 BUGS

We expect the server to talk first upon connection - This means, if you 
configure IPmanage to get the configuration from i.e. an HTTP port, or from any
port that waits for the client to start the conversation, we will B<just sit
waiting> until interrupted.

=head1 SEE ALSO

L<IPmanage::Conf>

=head1 AUTHOR

Gunnar Wolf <gwolf@gwolf.org>

=head1 COPYRIGHT

Copyright 2005 Gunnar Wolf / Instituto de Investigaciones Econmicas UNAM
This module is Free Software, it can be redistributed under the same terms 
as Perl.

=cut

use warnings;
use strict;
use Carp;
use Digest::MD5 qw(md5_hex);
use IO::Socket;
use IPmanage::Conf;
use Net::IP;

sub new {
    my ($self, $class, $db, $pass, $conf);
    $class = shift;
    $db = shift;
    $self = { db => $db, servers => [] };

    $conf = IPmanage::Conf->new($db);
    unless ($self->{pass} = $conf->get_val('server_notif_pass')) {
	$self->{pass} = ''; # avoid warnings
	carp "Could not get the server notification password - ".
	    "Continuing anyway, but expect problems...";
    }

    for my $addr (split /\s*,\s*/, $conf->get_val('server_notif_addr')) {
	my ($ip, $port, $ip_ck);
	# Check the address is well formed - 'a.b.c.d:e', where a.b.c.d is a
	# valid IP address and e is a valid TCP port
	unless (($ip,$port) = split(/:/, $addr) and
		$port =~ /^\d+$/ and $port > 0  and $port < 65536 and
		$ip =~ /^[\d\.]+$/ and $ip_ck = Net::IP->new($ip)) {
	    carp "Address $addr is not in a valid format";
	    return undef;
	}
	push @{$self->{servers}}, [$ip, $port];
    }

    bless $self, $class;

    return $self;
}

sub notify {
    my ($self, $sock, $conn_err);
    $self = shift;
    $conn_err = 0;

    # To notify a server, we just connect to the port specified in the 
    # configuration, get a challenge (this is, a text string created by the
    # remote server, compute the MD5 hash of this challenge string and the
    # specified password, and send this hash. 
    # The server will then take care of pulling the configuration from us via
    # the defined (probably HTTP) interface.
    for my $server (@{$self->{servers}}) {
	my ($sock, $challenge, $child);

	unless ($sock=IO::Socket::INET->new(PeerAddr => $server->[0],
					    PeerPort => $server->[1],
					    Proto => 'tcp')) {
	    carp "Could not connect to $server->[0]:$server->[1]: $!";
	    $conn_err = 1;
	    next;
	}

	$challenge = $sock->getline;
	chomp($challenge);

	$sock->print(md5_hex($challenge, $self->{pass}), "\n");
    }

    return $conn_err ? 0 : 1;
}

1;

# $Log$
# Revision 1.3  2005/06/29 00:08:21  gwolf
# Reports whether the connection appeared to be successful
#
# Revision 1.2  2005/05/17 01:19:58  gwolf
# Allow for gateways not to be located on the network boundary on dynamic networks
#
# Revision 1.1  2005/05/10 05:04:20  gwolf
# Introducing... IPmanage::Notif, for all of your notification needs!
#
